/*
Copyright (C) 2009-2011,2013 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
*/

package org.harctoolbox.guicomponents;

import java.io.File;
import org.python.core.PyException;
import org.python.core.PyObject;
import org.python.util.PythonInterpreter;

/**
 *
 */
public class JythonEngine /*implements Serializable*/ {

    private final PythonInterpreter python;
    //private int debug;
    //private home hm;
    //private jython_rl_completer completer;
    //private String initFile = "test.py";

    private String[] getstuff(String cmd, String prefix) {
        String[] s = null;
        try {
            s = (String[]) python.eval(cmd).__tojava__(Class.forName("[Ljava.lang.String;"));
        } catch (ClassNotFoundException ex) {
            // This cannot happen
        }
        if (prefix == null || s == null) {
        } else
            for (int i = 0; i < s.length; i++) {
                s[i] = prefix + s[i];
            }
        return s;
    }

    public JythonEngine(String initFile) throws PyException {
        PythonInterpreter.initialize(System.getProperties(), null, new String[0]);
        python = new PythonInterpreter();
        python.exec("import sys");

        //String harcinit = harcprops.get_instance().get_harcmacros();
        //python.exec("sys.path.append('" + harcprops.get_instance().get_pythonlibdir() + "')");
        try {
            if ((new File(initFile)).exists())
                python.execfile(initFile);
            else
                System.err.println("Python init file " + initFile + " does not exist");
        } catch (PyException e) {
            System.err.println("Initializing Jython failed, probably error in " + initFile + ".");
            //System.err.println(e.traceback.toString());
            //e.printStackTrace();
            throw (e);
        }
    }

    public boolean exec(String macro) {
        if (python !=  null) {
            try {
                python.exec(macro);
            } catch (Exception e) {
                System.err.println("Exception from Python " + e.getMessage());
                //e.printStackTrace();
            }
            return true;
        } else
            return false;
    }

    // The correctness of the return value is unclear to me.
    // At least it takes True to "true", False to "false", None to null.
    public String eval(String macro) {
        PyObject po = null;
        try {
            po = python.eval(macro);
        } catch (PyException pye) {
            System.err.println("Python error: " + pye.getMessage());
            //pye.printStackTrace();
            return null;
        }
        String s;
        try {
            s = po.asStringOrNull();
            //s = po.asString();
        } catch (Exception e) {
            //System.err.println("Buggers");
            s = Boolean.toString(po.__nonzero__());
        }
        return s;
    }

    public void execFile(String file) {
        python.execfile(file);
    }

    public void set(String name, int value) {
        python.set(name, value);
    }

    public void set(String name, Object object) {
        python.set(name, object);
    }

    public String get(String name) {
        return python.get(name).toString();
    }

    /*public void interact() {
        if (python.getClass() == harc_readline_jythonconsole.class)
            ((harc_readline_jythonconsole) python).interact();
        else
            System.err.println("Not interactive, returning...");
    }*/

    // TODO: should probably be more general
    public String[] getScrutletNames() {
        return getstuff("_harcfuncs_0_args_ok()", null);
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        JythonEngine jython = null;
        //try {
        jython = new JythonEngine("init.py");

        if (args.length > 0)
            //if (args[0].equals("-a"))
            //    harcutils.printtable("Macros with no or all defaulted arguments are:", jython.get_argumentless_macros());
            //else
            for (String arg : args) {
                jython.exec(arg);
        }
        //else
        //   jython.interact();
        //} catch (Exception e) {
        //System.err.println("Could not create jython engine.");
        //System.err.println(e.getMessage());
        //System.exit(IrpUtils.exitConfigReadError);
        //}

        //else
        //   jython.interact();
        //} catch (Exception e) {
        //System.err.println("Could not create jython engine.");
        //System.err.println(e.getMessage());
        //System.exit(IrpUtils.exitConfigReadError);
        //}
    }
}
